angular.module('app.directives', [])

.directive('blankDirective', [function(){

}])

.directive('srvtVisible', function () {
    return function (scope, element, attr) {
        scope.$watch(attr.srvtVisible, function (visible) {
            element.css('visibility', visible ? 'visible' : 'hidden');
        });
    };
})

.directive('srvtNumbersOnly', function () {
    return {
        require: 'ngModel',
        link: function (scope, element, attr, ngModelCtrl) {
            function fromUser(text) {
                if (text) {
                    var transformedInput = text.replace(/[^0-9]/g, '');

                    if (transformedInput !== text) {
                        ngModelCtrl.$setViewValue(transformedInput);
                        ngModelCtrl.$render();
                    }
                    return transformedInput;
                }
                return text;
            }            
            ngModelCtrl.$parsers.push(fromUser);
        }
    };
})

/* Accept Alpha Numeric Account no */
.directive('srvtAccountNumber', function () {
    return {
        require: 'ngModel',
        link: function (scope, element, attr, ngModelCtrl) {
            function fromUser(text) {
                if (text) {
                    var transformedInput = text.replace(/[^A-Za-z0-9]/g, '');
                    transformedInput = transformedInput.replace(/ /g, '');
                    if (transformedInput !== text) {
                        ngModelCtrl.$setViewValue(transformedInput);
                        ngModelCtrl.$render();
                    }
                    return transformedInput;
                }
                return text;
            }            
            ngModelCtrl.$parsers.push(fromUser);
        }
    };
})


// Taken from http://jsfiddle.net/gUSZS/
.directive('srvtMatch', function() {
    return {
        require: 'ngModel',
        link: function (scope, elem, attrs, model) {
            if (!attrs.srvtMatch) {
                console.error('srvtMatch expects a model as an argument!');
                return;
            }
            scope.$watch(attrs.srvtMatch, function (value) {
                // Only compare values if the second ctrl has a value.
                if (model.$viewValue !== undefined && model.$viewValue !== '') {
                    model.$setValidity('srvtMatch', value === model.$viewValue);
                }
            });
            model.$parsers.push(function (value) {
                // Mute the nxEqual error if the second ctrl is empty.
                if (value === undefined || value === '') {
                    model.$setValidity('srvtMatch', true);
                    return value;
                }
                var isValid = value === scope.$eval(attrs.srvtMatch);
                model.$setValidity('srvtMatch', isValid);
                return isValid ? value : undefined;
            });
        }
    };
})

// http://plnkr.co/edit/Y58zmJlD6wzfvs9NqgW9?p=preview
.directive('srvtDecimalAmount', function () {
    return {
        require: 'ngModel',
        link: function (scope, element, attr, ngModelCtrl) {

            var myAttr = attr;
            angular.element(element).on("blur", function(e) {
                var value = String(element[0].value);
                var floatValue = parseFloat(value).toFixed(2);
                ngModelCtrl.$setViewValue(floatValue);
                ngModelCtrl.$render();
            });

            function fromUser(amount) {
                var max = attr.max.split('.')[0];
                var maxLength = max.length;
                if (amount) {
                    var text = ngModelCtrl.$viewValue;
                    var wholeAmt = '';
                    var decimalAmt = '';
                    if (text.indexOf('.') >= 0) {
                        var arr = text.split('.');
                        wholeAmt = arr[0];
                        decimalAmt = '';
                        if (arr.length > 1)
                            decimalAmt = arr[1];
                    } else {
                        wholeAmt = text;
                        if (wholeAmt.length <= maxLength) {
                            return wholeAmt;
                        } else {
                            var truncatedValue = wholeAmt.substring(0, maxLength);
                            ngModelCtrl.$setViewValue(truncatedValue);
                            ngModelCtrl.$render();
                            return parseFloat(truncatedValue);
                        }
                    }

                    if (decimalAmt.length <= 2) {
                        return amount;
                    } else {
                        var truncatedValue = decimalAmt.substring(0, 2);
                        var newValue = wholeAmt + '.' + truncatedValue;
                        ngModelCtrl.$setViewValue(newValue);
                        ngModelCtrl.$render();
                        return parseFloat(newValue);
                    }
                }
                return undefined;
            }            
            ngModelCtrl.$parsers.push(fromUser);
        }
    };
})

.directive('srvtAmount', function () {
    return {
        require: 'ngModel',
        link: function (scope, element, attr, ngModelCtrl) {
            function fromUser(text) {
                if (text) {
                    var transformedInput = parseFloat(text);
                    if (!isNaN(transformedInput) && isFinite(text)) {
                        return text;
                    } else {
                        ngModelCtrl.$setViewValue(transformedInput);
                        ngModelCtrl.$render();
                        return undefined;
                    }
                    // var transformedInput = parseFloat(text).toString();

                    // if (transformedInput !== text) {
                    //     ngModelCtrl.$setViewValue(transformedInput);
                    //     ngModelCtrl.$render();
                    // }
                    return transformedInput;
                }
                return undefined;
            }            
            ngModelCtrl.$parsers.push(fromUser);
        }
    };
})

.directive('nksOnlyNumber', function () {
    return {
      restrict: 'EA',
      require: 'ngModel',
      link: function (scope, element, attrs, ngModel) {   
        scope.$watch(attrs.ngModel, function(newValue, oldValue) {
          var spiltArray = String(newValue).split("");
          if (spiltArray.length === 0) return;
          if (spiltArray.length === 1 
               && (spiltArray[0] == '-' 
               || spiltArray[0] === '.' )) return;
          if (spiltArray.length === 2 
               && newValue === '-.') return;

          /*Check it is number or not.*/
          if (isNaN(newValue)) {
               ngModel.$setViewValue(oldValue);
               ngModel.$render();
          }
        });
      }
    };
})

.directive('srvtFloat', function () {
    return {
      restrict: 'EA',
      require: 'ngModel',
      link: function (scope, element, attrs, ngModel) {   

        angular.element(element).on("keypress", function(e) {
          console.log ('value :' + this.value + ' e = ');
          console.log (e);
          var newValue = String(this.value) + e.key;

          var splitArray = newValue.split(".")
          console.log (splitArray);

          if (splitArray.length > 1 && splitArray[1].length > 2) {
            e.preventDefault();
          }

          if (isNaN(newValue) || !isFinite(newValue)) {
             e.preventDefault();
          }
        });
      }
    };
})

// https://jsfiddle.net/long2know/6ccrqa4s/?utm_source=website&utm_medium=embed&utm_campaign=6ccrqa4s

.directive('srvtDateBefore', function () {
    return {
        require: 'ngModel',
        link: function (scope, el, attrs, ctrl) {
            var isInclusive = attrs.dateOrEquals ? scope.$eval(attrs.dateOrEquals) : false,
                validate = function (val1, val2) {
                    if (val1 === undefined || val2 === undefined) return;
                    var isArray = val2 instanceof Array;
                    var isValid = true;
                    var date1 = new Date(val1);
                    if (isArray && val2.length > 0) {
                        for (var i = 0; i < val2.length; i++) {
                            if (val2[i] !== undefined) {
                                var date2 = new Date(val2[i]);
                                isValid = isValid && (isInclusive ? date1 <= date2 : date1 < date2);
                            }
                            if (!isValid)
                                break;
                        }
                    }
                    else {
                        if (val2 !== undefined) {
                            var date2 = new Date(val2);
                            date1.setHours(0,0,0,0);
                            date2.setHours(0,0,0,0);
                            isValid = isInclusive ? date1 <= date2 : date1 < date2;
                        }
                    }
                    ctrl.$setValidity('srvtDateBefore', isValid);
                };
            // Watch the value to compare - trigger validate()
            scope.$watch(attrs.srvtDateBefore, function () {
                validate(ctrl.$viewValue, scope.$eval(attrs.srvtDateBefore));
            });
 
            ctrl.$parsers.push(function (value) {
                validate(value, scope.$eval(attrs.srvtDateBefore));
                return value;
            })
        }
    }
})

.directive('srvtDateAfter', function () {
        return {
            require: 'ngModel',
            link: function (scope, el, attrs, ctrl) {
                var isInclusive = attrs.dateOrEquals ? scope.$eval(attrs.dateOrEquals) : false,
                    validate = function (val1, val2) {
                        if (val1 === undefined || val2 === undefined) return;
                        var isArray = val2 instanceof Array;
                        var isValid = true;
                        var date1 = new Date(val1);
                        if (isArray && val2.length > 0) {
                            for (var i = 0; i < val2.length; i++) {
                                if (val2[i] !== undefined) {
                                    var date2 = new Date(val2[i]);
                                    isValid = isValid && (isInclusive ? date1 >= date2 : date1 > date2);
                                }
                                if (!isValid)
                                    break;
                            }
                        }
                        else {
                            if (val2 !== undefined) {
                                var date2 = new Date(val2);
                                date1.setHours(0,0,0,0);
                                date2.setHours(0,0,0,0);
                                isValid = isInclusive ? date1 >= date2 : date1 > date2;
                            }
                        }
                        ctrl.$setValidity('srvtDateAfter', isValid);
                    };
                // Watch the value to compare - trigger validate()
                scope.$watch(attrs.srvtDateAfter, function () {
                    validate(ctrl.$viewValue, scope.$eval(attrs.srvtDateAfter));
                });

                ctrl.$parsers.unshift(function (value) {
                    validate(value, scope.$eval(attrs.srvtDateAfter));
                    return value;
                })
            }
        }
})

.directive ('atleastOne', function () {
    return {
        required: 'ngModel',
        link: function (scope, el, attrs, ctrl) {
            validate = function (val1, val2) {
                console.log ('In validate!');
            };

            scope.$watch(attrs.atleastOne, function (value) {
                validate (value, scope.$eval(attrs.atleastOne))
            });

            ctrl.$parsers.unshift(function (value) {
                validate(value, scope.$eval(attrs.atleastOne));
                return value;
            })
        }
    }
})

.directive('srvtFocusOn', function() {
   return function(scope, elem, attr) {
      scope.$on('focusOn', function(e, name) {
        if(name === attr.srvtFocusOn) {
          elem[0].focus();
        }
      });
   };
})

.directive('enterAsTab', function () {
    return function (scope, element, attrs) {
        element.bind("keydown keypress", function (e) {
            var code = e.keyCode || e.which;
            /* 13 = Enter, 9 = Tab */
            if(code === 13 || code === 9) {
                var tabindex = parseInt(element.attr('tabindex'));
                var inputArray = angular.element(document).find('input');
                if (tabindex != NaN) {
                    var elementToFocus = inputArray[tabindex + 1];
                    if(angular.isDefined(elementToFocus)) {
                        e.preventDefault();
                        elementToFocus.focus();
                    }
                }
            }
        });
    };
})

.directive('focusInvalid', ['$timeout', function ($timeout) {
    return {
        restrict: 'A',
        link: function (scope, elem) {

            // set up event handler on the form element
            elem.on('submit', function () {

                // find the first invalid element
                var firstInvalid = elem[0].querySelector('.ng-invalid');

                // if we find one, set focus
                if (firstInvalid) {
                        $timeout(function () {
                            firstInvalid.focus();
                        }, 0);
                }
            });
        }
    };
}])

.directive('stopEvent', function () {
  return {
    restrict: 'A',
    link: function (scope, element, attr) {
      element.bind('click', function (e) {
        e.stopPropagation();
      });
    }
  };
})

.directive('capitalize', function () {
    return {
        require: 'ngModel',
        link: function (scope, element, attrs, modelCtrl) {
            var capitalize = function (inputValue) {
                if (angular.isUndefined(inputValue))
                    return;

                var capitalized = inputValue.toUpperCase();
                if (capitalized !== inputValue) {
                    modelCtrl.$setViewValue(capitalized);
                    modelCtrl.$render();
                }
                return capitalized;
            }
            modelCtrl.$parsers.push(capitalize);
            capitalize(scope[attrs.ngModel]);  // capitalize initial value
        }
    };
});
;

